@@ -731,3 +731,141 @@ char *tt_gui = "#line quiet #port init gui 0\n"
 "}\n"
 
 "#delay 0 gui_init\n";
+
+
+char *tt_regex = "#event {PROGRAM START}\n"
+"{\n"
+"	#var tab {{i}{0}{0}{}{1}{}};\n"
+"	#info matches save;\n"
+"	draw_gui;\n"
+"	#config mouse on\n"
+"}\n"
+"\n"
+"#event {SCREEN RESIZE}\n"
+"{\n"
+"	draw_gui;\n"
+"	#buffer end\n"
+"}\n"
+"\n"
+"#alias {draw_gui}\n"
+"{\n"
+"	#screen clear scroll;\n"
+"	#screen scroll 7 1 -4 -1;\n"
+"	#draw jade Silver rounded talign box 1 1 3 -1;\n"
+"	#draw jade Azure scaled tile 1 3 1 6 {REGULAR EXPRESSION};\n"
+"	#draw jade Silver rounded talign box 4 1 6 -1;\n"
+"	#draw jade Azure scaled tile 4 3 4 6 {TEST STRING};\n"
+"	#draw jade Green rounded calign box -3 -6 -1 -1 {exit};\n"
+"	#draw jade Green rounded calign box -3 -12 -1 -7 {help};\n"
+"\n"
+"	#screen inputregion 5 2 5 -2 TAB_2;\n"
+"	next_tab;\n"
+"	next_tab\n"
+"}\n"
+"\n"
+"#alias {next_tab}\n"
+"{\n"
+"	#if {$tab[i] == 0}\n"
+"	{\n"
+"		#screen inputregion 5 2 5 -2 TAB_2;\n"
+"		#cursor clear;\n"
+"		#cursor set $tab[1];\n"
+"	};\n"
+"	#if {$tab[i] == 1}\n"
+"	{\n"
+"		#screen inputregion 2 2 2 -2 TAB_1;\n"
+"		#cursor clear;\n"
+"		#cursor set $tab[0];\n"
+"	};\n"
+"	#math tab[i] ($tab[i] + 1) % 2;\n"
+"}\n"
+"\n"
+"#alias {mouse_tab}\n"
+"{\n"
+"	#var tab[i] %1;\n"
+"	#if {%1 == 0}\n"
+"	{\n"
+"		#screen inputregion 2 2 2 -2;\n"
+"		#cursor clear;\n"
+"		#cursor set $tab[0]\n"
+"	};\n"
+"	#if {%1 == 1}\n"
+"	{\n"
+"		#screen inputregion 5 2 5 -2;\n"
+"		#cursor clear;\n"
+"		#cursor set $tab[1]\n"
+"	};\n"
+"	#cursor position %2\n"
+"}\n"
+"\n"
+"#button {2;2;2;-2;PRESSED MOUSE BUTTON ONE} {mouse_tab 0 %3-1}\n"
+"#button {5;2;5;-2;PRESSED MOUSE BUTTON ONE} {mouse_tab 1 %3-1}\n"
+"#button {-3;-6;-1;-1} {#end}\n"
+"#button {-3;-12;-1;-7} {#buffer clear;#help regex}\n"
+"\n"
+"#event {SCROLLED MOUSE WHEEL DOWN} {#buffer down 3}\n"
+"#event {SCROLLED MOUSE WHEEL UP} {#buffer up 3}\n"
+"\n"
+"#macro {\t}\n"
+"{\n"
+"	next_tab\n"
+"}\n"
+"\n"
+"#alias {matches}\n"
+"{\n"
+"	#var result {};\n"
+"\n"
+"	#foreach {*info[MATCHES][]} {index}\n"
+"	{\n"
+"		#format temp {\\n%+3s %s} {<138>%$index} {<178>$info[MATCHES][$index]};\n"
+"		#cat result {$temp}\n"
+"	};\n"
+"	#if {{$last} === {$result}}\n"
+"	{\n"
+"		#return\n"
+"	};\n"
+"	#buffer clear;\n"
+"	#screen clear scroll;\n"
+"	#line ignore #showme {$result};\n"
+"\n"
+"	#var last {$result}\n"
+"}\n"
+"\n"
+"#event {GAG NO SESSION ACTIVE} {#nop}\n"
+"\n"
+"#event {PROCESSED KEYPRESS}\n"
+"{\n"
+"	#cursor get temp;\n"
+"	#replace temp {\\\\x7B%*\\\\x7D} {{&1}};\n"
+"	#replace temp {\\\\x7B%*\\\\x7D} {{&1}};\n"
+"\n"
+"	#if {{$temp} !== {$tab[$tab[i]]}}\n"
+"	{\n"
+"		#cursor get tab[$tab[i]];\n"
+"		#delay {input} {input_update} {0.1}\n"
+"	}\n"
+"}\n"
+"\n"
+"#alias {input_update}\n"
+"{\n"
+"	#replace tab[0] {\\\\x7B%*\\\\x7D} {{&1}};\n"
+"	#replace tab[0] {\\\\x7B%*\\\\x7D} {{&1}};\n"
+"	#replace tab[1] {\\\\x7B%*\\\\x7D} {{&1}};\n"
+"	#replace tab[1] {\\\\x7B%*\\\\x7D} {{&1}};\n"
+"\n"
+"	#regex {$tab[1]} {$tab[0]}\n"
+"	{\n"
+"		#info matches save\n"
+"	}\n"
+"	{\n"
+"		#if {{$last} !== {}}\n"
+"		{\n"
+"			#buffer clear;\n"
+"			#screen clear scroll;\n"
+"			#line ignore #showme {<118>The regular expression does not match the test string.\n};\n"
+"			#var last {}\n"
+"		};\n"
+"		#return\n"
+"	};\n"
+"	matches\n"
+"}\n";
