@@ -24,7 +24,6 @@
 ******************************************************************************/
 
 #include <sys/types.h>
-#include <pcre.h>
 
 #include "tintin.h"
 
@@ -90,9 +89,9 @@ DO_COMMAND(do_regexp)
 	return ses;
 }
 
-int regexp_compare(struct session *ses, pcre *nodepcre, char *str, char *exp, int comp_option, int flag)
+int regexp_compare(struct session *ses, pcre2_code *nodepcre, char *str, char *exp, int comp_option, int flag)
 {
-	pcre *regex;
+	pcre2_code *regex;
 	int i, j, matches;
 
 	if (nodepcre == NULL)
@@ -109,18 +108,28 @@ int regexp_compare(struct session *ses, pcre *nodepcre, char *str, char *exp, in
 		return FALSE;
 	}
 
-	matches = pcre_exec(regex, NULL, str, strlen(str), 0, 0, gtd->match, 303);
+	pcre2_match_data *match_data = pcre2_match_data_create(101, NULL);
+
+	matches = pcre2_match(regex, (PCRE2_SPTR) str, strlen(str), 0, 0, match_data, NULL);
 
 	if (matches <= 0)
 	{
 		if (nodepcre == NULL)
 		{
-			free(regex);
+			pcre2_code_free(regex);
 		}
+		pcre2_match_data_free(match_data);
+
 		return FALSE;
 	}
 
-	// REGEX_FLAG_FIX handles %1 to %99 usage. Backward compatibility.
+	PCRE2_SIZE *ovector = pcre2_get_ovector_pointer(match_data);
+
+	memcpy(gtd->match, ovector, matches * 2 * sizeof(PCRE2_SIZE));
+
+	pcre2_match_data_free(match_data);
+
+	// REGEX_FLAG_FIX handles %1 to %99 usage. Backward compatible.
 
 	switch (flag)
 	{
@@ -183,41 +192,42 @@ int regexp_compare(struct session *ses, pcre *nodepcre, char *str, char *exp, in
 
 	if (nodepcre == NULL)
 	{
-		free(regex);
+		pcre2_code_free(regex);
 	}
 
 	return TRUE;
 }
 
-pcre *regexp_compile(struct session *ses, char *exp, int comp_option)
+pcre2_code *regexp_compile(struct session *ses, char *exp, int comp_option)
 {
-	const char *error;
-	int i;
+	int errorcode;
+	PCRE2_SIZE erroroffset;
 /*
 	if (HAS_BIT(ses->charset, CHARSET_FLAG_UTF8))
 	{
-		comp_option |= PCRE_UTF8|PCRE_NO_UTF8_CHECK;
+		comp_option |= PCRE2_UTF|PCRE2_NO_UTF_CHECK|PCRE2_UCP;
 	}
 */
-	return pcre_compile(exp, comp_option, &error, &i, NULL);
+	pcre2_code *regex = pcre2_compile((PCRE2_SPTR) exp, PCRE2_ZERO_TERMINATED, comp_option, &errorcode, &erroroffset, NULL);
+/*
+	if (regex)
+	{
+		pcre2_jit_compile(regex, PCRE2_JIT_COMPLETE);
+	}
+*/
+	return regex;
 }
 
 
 
-
-/******************************************************************************
-* Calls tintin_regexp checking if the string matches, and automatically fills *
-* in the text represented by the wildcards on success.                        *
-******************************************************************************/
+// Check if the string matches and fill in wildcards on success
 
 int check_one_regexp(struct session *ses, struct listnode *node, char *line, char *original, int comp_option)
 {
-	char *exp, *str;
+	char *exp, *str, result[BUFFER_SIZE];
 
 	if (node->regex == NULL)
 	{
-		char result[BUFFER_SIZE];
-
 		substitute(ses, node->arg1, result, SUB_VAR|SUB_FUN);
 
 		exp = result;
@@ -235,14 +245,13 @@ int check_one_regexp(struct session *ses, struct listnode *node, char *line, cha
 	else
 	{
 		str = line;
-	}	
+	}
 
 	return tintin_regexp(ses, node->regex, str, exp, comp_option, REGEX_FLAG_ARG);
 }
 
-/*
-	Keep synched with tintin_regexp and tintin_regexp_compile
-*/
+// Keep synched with tintin_regexp and tintin_regexp_compile
+
 
 int get_regex_range(char *in, char *out, int *var, int *arg)
 {
@@ -465,7 +474,7 @@ int tintin_regexp_check(struct session *ses, char *exp)
 //    3.4 If FIX is set the gtd->args index is used and valid tinexp is assumed
 //    3.4 return TRUE
 
-int tintin_regexp(struct session *ses, pcre *nodepcre, char *str, char *exp, int comp_option, int flag)
+int tintin_regexp(struct session *ses, pcre2_code *nodepcre, char *str, char *exp, int comp_option, int flag)
 {
 	char out[BUFFER_SIZE], *pti, *pto;
 	int i, arg = 1, var = 1, fix = 0;
@@ -510,7 +519,7 @@ int tintin_regexp(struct session *ses, pcre *nodepcre, char *str, char *exp, int
 			case '\\':
 				if (pti[1] == 'n')
 				{
-					SET_BIT(comp_option, PCRE_MULTILINE);
+					SET_BIT(comp_option, PCRE2_MULTILINE);
 				}
 				else if (pti[1] == 0)
 				{
@@ -793,6 +802,10 @@ int tintin_regexp(struct session *ses, pcre *nodepcre, char *str, char *exp, int
 
 							case '{':
 								pti = get_arg_in_braces(ses, pti+2, pto, GET_ALL);
+								if (strstr(pto, "\\n"))
+								{
+									SET_BIT(comp_option, PCRE2_MULTILINE);
+								}
 								pto += strlen(pto);
 								break;
 
@@ -818,7 +831,7 @@ int tintin_regexp(struct session *ses, pcre *nodepcre, char *str, char *exp, int
 	return regexp_compare(ses, nodepcre, str, out, comp_option, flag + fix);
 }
 
-pcre *tintin_regexp_compile(struct session *ses, struct listnode *node, char *exp, int comp_option)
+pcre2_code *tintin_regexp_compile(struct session *ses, struct listnode *node, char *exp, int comp_option)
 {
 	char out[BUFFER_SIZE], *pti, *pto;
 
@@ -873,7 +886,7 @@ pcre *tintin_regexp_compile(struct session *ses, struct listnode *node, char *ex
 				}
 				else if (pti[1] == 'n')
 				{
-					SET_BIT(comp_option, PCRE_MULTILINE);
+					SET_BIT(comp_option, PCRE2_MULTILINE);
 					SET_BIT(node->flags, NODE_FLAG_MULTI);
 				}
 				else if (pti[1] == 0)
@@ -901,6 +914,7 @@ pcre *tintin_regexp_compile(struct session *ses, struct listnode *node, char *ex
 					}
 					if (pto[0] == '\\' && pto[1] == 'n')
 					{
+						SET_BIT(comp_option, PCRE2_MULTILINE);
 						SET_BIT(node->flags, NODE_FLAG_MULTI);
 					}
 					pto++;
@@ -1157,6 +1171,7 @@ pcre *tintin_regexp_compile(struct session *ses, struct listnode *node, char *ex
 									}
 									if (pto[0] == '\\' && pto[1] == 'n')
 									{
+										SET_BIT(comp_option, PCRE2_MULTILINE);
 										SET_BIT(node->flags, NODE_FLAG_MULTI);
 									}
 									pto++;
